/* analyses impulse functions and generates responses, etc */

#include <stdio.h>
#include "kernel.h"
#include <math.h>
#include <swis.h>

FILE* infile;
FILE* outfile;

double sample[5000], imp0[5000], imp1[5000], imp2[5000];

double I[4097], Q[4097], Amp[4097], Ang[4097];

double t0,f0,w0,pi2,wnow,fnow,tstep,tnow;

double rad_to_deg;


int n,nmax;

void readin(void);

void transform(double*);

void transform_and_save(double*,int);

int main(void)
{
  
  nmax = 4096;  pi2 = 2.0*3.1415927;
  tstep = 1.0/44100.0;
  
  rad_to_deg = 180.0/3.1415927;
  
  readin();
  
  n=0;
  do
  {
    printf("%lf,  %lf,  %lf, %lf \n",sample[n],imp0[n],imp1[n],imp2[n]);
    n++;
  } while (n < 4);
  
  t0 = 4096.0/44100.0;
  f0 = 1.0/t0;  w0 = f0*pi2;
  printf("FT span = %7.4lf ms  f0 = %7.4lf\n",t0*1000.0,f0);
  
  
  
  transform_and_save(imp0,0);
  
  
  transform(imp1);
  
  printf("\n\n");
  
  transform_and_save(imp1,1);
  
  
  transform(imp2);
  
  printf("\n\n");
  
  transform(imp0);
  
  
}

void transform(double* inarray)
{
  
  double tp, phi, nscale;
  int p;
  
  wnow = w0; fnow = f0; n = 0; nscale = 1.0/(4096.0*2048.0);
  
  I[p] = 0.0;  Q[p] = 0.0;
  
  p=0;
  
  do
  {
    n=0;
    
    do
   {
     tp = tstep*n;   phi=wnow*tp; 
    
     I[p] += inarray[n] * cos(phi);  Q[p] += inarray[n] * sin(phi);
   
     n++;
   
   } while( n < 4096);
 
 
   I[p] = nscale*I[p]; Q[p] = nscale*Q[p];
   
   Amp[p]   = sqrt( I[p]*I[p] + Q[p]*Q[p] );
   Ang[p] = rad_to_deg * atan2(Q[p],I[p]);
 
   printf("result: %8.4lf, %10.7lf, %10.7lf, %10.7lf, %10.6lf\n",fnow,I[p],Q[p],Amp[p],Ang[p]);
    
    p += 1;
    
    wnow += w0; fnow += f0;
  } while (p<20);

}

void transform_and_save(double* inarray, int filenumber)
{
  char filename[1024];
  double tp, phi, nscale, stitch;
  int p;
  
  sprintf(filename,"RAM::RamDisc0.$.Output%d",filenumber);
  
  outfile=fopen(filename,"wt");
  
  wnow = w0; fnow = 0.0; n = 0; nscale = 1.0/(4096.0*8.0);
  
  I[p] = 0.0;  Q[p] = 0.0; stitch = 0.0;
  
  p=0;
  
  do
  {
    n=0;
    
    do
   {
     tp = tstep*n;   phi=wnow*tp; 
    
     I[p] += inarray[n] * cos(phi);  Q[p] += inarray[n] * sin(phi);
   
     n++;
   
   } while( n < 4096);
 
 
   I[p] = nscale*I[p]; Q[p] = nscale*Q[p];
   
   Amp[p]   = sqrt( I[p]*I[p] + Q[p]*Q[p] );
   Ang[p] = rad_to_deg * atan2(Q[p],I[p]) + stitch;
   
 
   
   
 
   fprintf(outfile," %8.4lf, %10.7lf, %10.7lf, %10.7lf, %10.6lf\n",fnow,I[p],Q[p],Amp[p],Ang[p]);
    
    p += 1;
    
    wnow += w0; fnow += f0;
  } while (fnow < 22000.0);

  fclose(outfile);

}
void readin(void)
{
  double rs,r0,r1,r2;
  int ir;
  infile = fopen("RAM::RamDisc0.$.Trio/csv","rt");
  
  ir=0;
  do
  {
    fscanf(infile,"%lf,%lf,%lf,%lf",&rs,&r0,&r1,&r2);
    sample[ir] = rs;  imp0[ir] = r0;  imp1[ir] = r1; imp2[ir] = r2;
    ++ir;
  } while (ir <= nmax+1);
  fclose(infile);
  
}
